// MARK: Imports
import path from 'node:path';
import __path from "./handlers/path.js";
import log from "./handlers/log.js";
import config from './handlers/config.js';

// Express & more
import express from 'express';
import cors from 'cors'
import session from 'express-session';
import fileUpload from 'express-fileupload';
import bodyParser, { Options } from 'body-parser';
import { Eta } from "eta";
import passport from 'passport';

import routes from './routes/index.js';

log.core.trace("Running from path: " + __path);


// MARK: Express
const app = express();

// TODO: Version check need to be rewritten.
//app.locals.versionRevLong = require('child_process').execSync('git rev-parse HEAD').toString().trim();
//app.locals.versionRev = require('child_process').execSync('git rev-parse --short HEAD').toString().trim();
//app.locals.versionRevLatest = require('child_process').execSync('git ls-remote --refs -q').toString().trim().split('\t')[0];
app.locals.versionRev = '0';
app.locals.versionRevLong = '0';
app.locals.versionRevLatest = '0';

if (app.locals.versionRevLong === app.locals.versionRevLatest) {
	log.core.info(`Running Latest Version (${app.locals.versionRevLong})`);
} else {
	log.core.info(`Running Version: ${app.locals.versionRevLong} (Latest: ${app.locals.versionRevLatest})`);
}

// ETA Init
const eta = new Eta({ views: path.join(__path, "views") })
app.engine("eta", buildEtaEngine())
app.set("view engine", "eta")


// MARK: Express Middleware & Config
app.set('x-powered-by', false);

app.use(fileUpload());
//app.use(cors());
app.use(bodyParser.urlencoded({ extended: false }));
app.use(bodyParser.json());

// Session store
// TODO: Move secret to config -> Autogenerate.
app.use(
	session({
		secret: config.global.auth.cookie_secret,
		resave: false,
		saveUninitialized: false,
		cookie: { secure: config.global.auth.cookie_secure }
	})
);
app.use(passport.authenticate('session'));
app.use(routes);


app.listen(config.global.http_port, config.global.http_listen_address, () => {
	log.web.info(`Listening at http://${config.global.http_listen_address}:${config.global.http_port}`);
});



// MARK: Helper Functions
function buildEtaEngine() {
	return (path:string, opts:Options, callback: CallableFunction) => {
		try {
			const fileContent = eta.readFile(path);
			const renderedTemplate = eta.renderString(fileContent, opts);
			callback(null, renderedTemplate);
		} catch (error) {
			callback(error);
		}
	};
}

